#include "stm32f4xx_hal.h"
#include "stm32f4xx_hal_dma.h"
#include "stm32f4xx_hal_uart.h"
#include "stm32f4xx_hal_usart.h"
#include "system_stm32f4xx.h"


volatile int blink;

/* method to initilaize the LED on PA5 GPIO pin*/
void InitializeLEDs(void)
{
	RCC->AHB1ENR 	|= (RCC_AHB1ENR_GPIOAEN); // enable clocks
	GPIOA->MODER 	|= (0x1 << 10); // set port A pin 5 output
  	GPIOA->OSPEEDR 	|= (0x1 << 11); // set port A pin 5 output speed fast
  	GPIOA->PUPDR 	|= (0x1 << 10); // set port A pin 5 pull-up
}

/* method to initialize the serial port interface */
void InitializeSerial(int baudrate){
     /* Initialize USART pins */
	RCC->AHB1ENR |= (RCC_AHB1ENR_GPIOAEN);  // enable clocks 

	GPIOA->MODER = (GPIOA->MODER & 0xffffff0f) | 0xA0;	//set PA2 and PA3 to alternate function '10'
	GPIOA->OTYPER = (GPIOA->OTYPER & 0xfffffff3);		// set push-pull register
	GPIOA->PUPDR = (GPIOA->PUPDR & 0xffffff0f);			// set no pull-up
	GPIOA->AFR[0] = (GPIOA->AFR[0] & 0xffff77ff) | 0x7700;	// set alternate function to USART2
	
	/* Initialize USART core */
	RCC->APB1ENR |= (RCC_APB1ENR_USART2EN); // enable clocks
  
    USART2->CR1 &= ~USART_CR1_UE; // temporarily disable the core
    USART2->CR1 = (uint32_t)(USART_WORDLENGTH_8B | USART_PARITY_NONE | USART_MODE_TX_RX); // set parameters
   
    USART2->CR2 = (uint32_t)(USART_CLOCK_ENABLED | UART_STOPBITS_1);
    USART2->CR2 &= ~(USART_CR2_LINEN | USART_CR2_CLKEN);
	
	USART2->CR3 &= ~(USART_CR3_SCEN | USART_CR3_HDSEL | USART_CR3_IREN);
    USART2->BRR = __USART_BRR(HAL_RCC_GetPCLK1Freq(), baudrate);  // set baudrate
    USART2->CR1 |=  USART_CR1_UE;  	// enable the core
}

/* method to initilaize the timer2 */
void InitializeTimer()
{
	RCC->APB1ENR |= (RCC_APB1ENR_TIM2EN); // enable clocks

	TIM2->CR1 |= 0x1;   // Counter enable bit to start the timer
	TIM2->PSC  = 40000; // Prescale register
	TIM2->ARR  = 400;   // auto-reload register - the reset value of the counter

}

/* method to enable interrupts for timer 2 */
void EnableTimerInterrupt() {
	TIM2->DIER |= (0x1);  //enable interrupt
   	NVIC_EnableIRQ(TIM2_IRQn);	//enable interrupts from timer2
}

/* method to disable interrupts for timer 2 */
void DisableTimerInterrupt() {
	TIM2->DIER &= ~(0x1);  //enable interrupt
   	NVIC_DisableIRQ(TIM2_IRQn);	//enable interrupts from timer2
}

/* Interrupt handler routine for timer 2 */ 
extern void TIM2_IRQHandler()
{
	if (((TIM2->SR) & 0x01) == 1){	//read timer status register
    	TIM2->SR   &= ~(0x1);	//clear timer status register
    	
    	if(blink){
	    	GPIOA->ODR ^= (0x1 << 5); 	//blink the led
    	}
    }
}

/* method to delay by busy loop */
void ms_delay(int ms) {
  while (ms-- > 0) {
    volatile int x = 5971;
    while (x-- > 0)
      __asm("nop"); // no operation
  }
}

/* main method */ 
int main()
{
	/* update frequency of the STM board to 160 MHz */
   	SystemClock_Config();
   	SystemCoreClockUpdate();
   	
   	/* Initialize PA5 LED GPIO pin */
	InitializeLEDs();
	/* Initialize serial port */
	InitializeSerial(115200);
	/* Initialize timer for LED blinking and enable interrupts */
	InitializeTimer();
	EnableTimerInterrupt();
	
	/* blink the LED 5 times */
	GPIOA->ODR |= (1 << 5);	//LED_on
	int i;
    for (i = 0; i < 5*2; ++i) { 
    	ms_delay(100);
    	GPIOA->ODR ^= (1 << 5); //toggle LED
    }

	/* Send character i */
	USART2->DR |= 'i';

	int ok = 1;
	uint16_t periods[5] = { 200, 400, 800, 2000, 4000 };
	blink = 0;
	
	/* loop to receive from UART and blink the LED accordingly */
	while(1){
		while((USART2->SR & (0x01 << 5)) == 0){} // active wait for received byte
 	
 		ok = 1;
 		char c = USART2->DR; // reading received byte
 		switch(c){	
			case 's':
				blink = 0;
				GPIOA->ODR |= (1 << 5);		//LED - on
				break;
 			case 'e':
				blink = 0;
	 			GPIOA->ODR &= ~(1 << 5);	//LED - off
	 			break;	 			
	 		default: 
	 			if (c >= '1' && c <= '5') {
	 				blink = 1;
	 				TIM2->ARR = periods[c - '1'];	//update period counter
	 				TIM2->EGR = 0x1;	//reset timer
        		} else {
           			ok = 0;
        		}		
	 			break;
 		}
 		while((USART2->SR & (0x01 << 7)) == 0){} // active wait to allow sending
 		USART2->DR =(ok ? 'a' : 'f'); // send response
	}
}